from requests import get, post
import json


class Sessions(object):

    headers = {
        "Origin": "https://www.joox.com",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36",
    }

    def search(self, q, type, limit, offset):
        params = dict(search_input=q, sin=offset, ein=limit, type=type)

        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_SEARCH_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to get search request.")
        return json.loads(r.text)

    def _get(self, url, params={}, withAuth=False):
        if withAuth:
            auth = self.getUserInfo()
            params.update(
                dict(
                    wmid=auth["wmid"],
                    s=auth["session_key"],
                    u=auth["user_type"],
                    c=auth["reg_country"],
                )
            )
        params.update(self.DEFAULT_PARAMS)
        data = get(url, params=params, headers=self.headers)
        return data

    def _post(self, url, params={}, payload={}, files=None):
        auth = self.getUserInfo()
        params = dict(
            wmid=auth["wmid"],
            s=auth["session_key"],
            u=auth["user_type"],
            c=auth["reg_country"],
        )
        payload.update({"wmid": auth["wmid"], **self.DEFAULT_PARAMS})
        data = post(url, params=params, data=json.dumps(payload), headers=self.headers)
        return data



from .config import Config
from .client import Client
from .user import User
from .sessions import Sessions
from .utils import *
from datetime import datetime
import requests, json, os


class JooxAuthError(Exception):
    pass


class Jooxy(Config, Client, Sessions, User):

    DEFAULT_PARAMS = dict(country="id", lang="en")
    isLogin = False
    cache = dict()

    def __init__(self, email=None, password=None, saveData=True, cachePath="data/"):
        self.saveData = saveData
        if email and password:
            self._email = email
            self._password = password
            if cachePath[-1] != "/":
                cachePath += "/"
            if not os.path.isdir(cachePath):
                os.makedirs(cachePath)
            self._cachePath = cachePath
            self.isLogin = True
            self.getUserInfo()

    def isSessionExpired(self, userInfo):
        now = datetime.today()
        expiredTime = "".join(userInfo["expire_time"].split()[1:]).replace("GMT", "")
        expireSession = datetime.strptime(expiredTime, "%d%b%Y%H:%M:%S")
        return expireSession <= now

    def getCachedUserInfo(self):
        if self.saveData:
            with open(
                "{}{}.json".format(self._cachePath, self._email.split("@")[0])
            ) as f:
                userInfo = json.loads(f.read())
        else:
            userInfo = self.cache
        if "expire_time" in userInfo:
            if self.isSessionExpired(userInfo):
                log("Session key is expired, trying to re-login..")
                if self.saveData:
                    os.remove(
                        "{}{}.json".format(self._cachePath, self._email.split("@")[0])
                    )
                else:
                    self.cache = dict()
                log("Removing old user info..")
                return self.getUserInfo()
            return userInfo
        return None

    def _saveUserInfo(self, userInfo):
        try:
            if self.saveData:
                with open(
                    "{}{}.json".format(self._cachePath, self._email.split("@")[0]), "w"
                ) as f:
                    f.write(json.dumps(userInfo, indent=4))
                    log("User info file has been saved!")
                    log("Welcome {}".format(userInfo["nickname"]))
            else:
                self.cache.update(userInfo)
                log("User info has been saved in sessions!")
                log("Welcome {}".format(userInfo["nickname"]))
        except IOError:
            log("Failed to save user info")

    def getUserInfo(self):
        if self.cache != {}:
            return self.getCachedUserInfo()
        if os.path.exists(
            "{}{}.json".format(self._cachePath, self._email.split("@")[0])
        ):
            return self.getCachedUserInfo()
        newPassword = encryptPassword(self._password)
        params = dict(
            callback="callBackEmailAuth",
            authtype=2,
            wxopenid=self._email,
            password=newPassword,
        )
        params.update(self.DEFAULT_PARAMS)
        r = requests.get(self.JOOX_API_DOMAIN + self.JOOX_AUTH_PATH, params=params)
        if r.status_code != 200:
            raise JooxAuthError(r.reason)
        userInfo = r.json()
        if "nickname" in userInfo:
            self._saveUserInfo(userInfo)
            return userInfo
        return None



class Config(object):

    JOOX_API_DOMAIN = "https://api-jooxtt.sanook.com/web-fcgi-bin"
    JOOX_AUTH_PATH = "/web_wmauth"
    JOOX_GETFAV_PATH = "/web_getfav"
    JOOX_ADD_DIR_PATH = "/web_fav_add_dir"
    JOOX_DEL_DIR_PATH = "/web_fav_del_dir"
    JOOX_ADD_SONG_PATH = "/web_fav_add_song"
    JOOX_DEL_SONG_PATH = "/web_fav_del_song"
    JOOX_SEARCH_PATH = "/web_search"
    JOOX_TAG_LIST_PATH = "/web_tag_list"
    JOOX_RECOMMENDED_MORE_PATH = "/web_recommend_more"
    JOOX_GET_DISS_PATH = "/web_get_diss"
    JOOX_GET_ALBUMINFO_PATH = "/web_get_albuminfo"
    JOOX_HOT_QUERY_PATH = "/web_hot_query"
    JOOX_GET_TOPLIST_PATH = "/web_get_toplist"
    JOOX_TOPLIST_DETAIL_PATH = "/web_toplist_detail"
    JOOX_ALL_SINGER_LIST_PATH = "/web_all_singer_list"
    JOOX_SINGER_CATEGORY_PATH = "/web_singer_category"
    JOOX_ALBUM_SINGER_PATH = "/web_album_singer"
    JOOX_LYRIC_PATH = "/web_lyric"
    JOOX_GET_SONGINFO_PATH = "/web_get_songinfo"



import json


class Client(object):
    def search(self, q, type, limit, offset):
        params = dict(search_input=q, sin=offset, ein=limit, type=type)
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_SEARCH_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def searchTracks(self, q, limit=30, offset=0):
        return self.search(q, None, limit, offset)

    def searchArtists(self, q, limit=30, offset=0):
        return self.search(q, 2, limit, offset)

    def searchAlbums(self, q, limit=30, offset=0):
        return self.search(q, 1, limit, offset)

    def searchPlaylist(self, q, limit=30, offset=0):
        return self.search(q, 3, limit, offset)

    def getPlaylistTaglist(self):
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_TAG_LIST_PATH)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getAllPlaylist(self, tagId=None, limit=50, offset=0):
        params = dict(sin=offset, ein=limit, req_type=5, tag_id=tagId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_RECOMMENDED_MORE_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getCategoryPlaylist(self, tagId, limit=50, offset=0):
        return self.getAllPlaylist(tagId, limit, offset)

    def getPlaylist(self, playlistId):
        params = dict(qryDissID=playlistId)
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_GET_DISS_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getAllAlbum(self, limit=50, offset=0):
        params = dict(sin=offset, ein=limit, req_type=4)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_RECOMMENDED_MORE_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getAlbumInfo(self, albumId):
        params = dict(all=1, albumid=albumId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_GET_ALBUMINFO_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getHotTracksAndArtists(self):
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_HOT_QUERY_PATH)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getTopChartList(self):
        params = dict(song_num=1)

        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_GET_TOPLIST_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getTopChart(self, topId=33, limit=100, offset=0):
        params = dict(sin=offset, ein=limit, topid=topId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_TOPLIST_DETAIL_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getAllArtist(self, limit=49, offset=0):
        params = dict(sin=offset, ein=limit, is_all=1)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_ALL_SINGER_LIST_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getArtistCategory(self):
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_SINGER_CATEGORY_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getCategoryArtists(self, categoryId, limit=49, offset=0):
        params = dict(sin=offset, ein=limit, category_id=categoryId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_ALL_SINGER_LIST_PATH, params=params
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getArtistTracks(self, singerId, limit=29, offset=0):
        params = dict(sin=offset, ein=limit, cmd=2, singerid=singerId)
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_ALBUM_SINGER_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getArtistAlbums(self, singerId, limit=29, offset=0):
        params = dict(sin=offset, ein=limit, cmd=1, singerid=singerId)
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_ALBUM_SINGER_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    def getTrackLyric(self, musicId):
        params = dict(musicid=musicId)
        r = self._get(self.JOOX_API_DOMAIN + self.JOOX_LYRIC_PATH, params=params)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)


__copyright__ = "Copyright 2019 by Ervan R.F"
__version__ = "1.0.4"
__license__ = "BSD-3-Clause"
__author__ = "Ervan R.F"
__author_email__ = "ervanroot@gmail.com"
__url__ = "http://github.com/ervan0707/jooxy"

from .auth import Jooxy
from .config import Config
from .client import Client
from .user import User
from .sessions import Sessions


"""
Unofficial Joox Web API
=========================================

    >>> from jooxy import *

Links
`````

* `GitHub repository <https://github.com/ervan0707/jooxy>`_

"""
from __future__ import with_statement
import re, codecs

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools

    use_setuptools()
    from setuptools import setup


with open("README.rst") as f:
    setup(
        name="jooxy",
        packages=["jooxy"],
        version="1.0.4",
        license="BSD 3 Clause License",
        author="Ervan R.F",
        author_email="ervanroot@gmail.com",
        url="https://github.com/ervan0707/jooxy",
        description=" Unofficial Joox Web API",
        long_description=f.read(),
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Console",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.1",
            "Programming Language :: Python :: 3.2",
            "Programming Language :: Python :: 3.3",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        ],
        install_requires=["requests"],
    )


import json, base64


def loggedIn(func):
    def checkLogin(*args, **kwargs):
        if args[0].isLogin:
            return func(*args, **kwargs)
        else:
            raise Exception("Login required.")

    return checkLogin


class User(object):
    @loggedIn
    def getAllMyPlaylist(self):
        params = dict(reqtype=4, inner=1)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_GETFAV_PATH, params=params, withAuth=True
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def getMyPlaylist(self, myPlaylistId):
        params = dict(reqtype=3, inner=1, listid=myPlaylistId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_GETFAV_PATH, params=params, withAuth=True
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def createPlaylist(self, playlistName):
        playlistName = base64.b64encode(bytes(playlistName, "utf-8")).decode("utf-8")
        payload = dict(
            channel_id=1,
            from_type=3,
            items=[dict(gt=0, gl="", mv=0, dv=0, fn=playlistName)],
        )
        r = self._post(self.JOOX_API_DOMAIN + self.JOOX_ADD_DIR_PATH, payload=payload)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def removeMyPlaylist(self, myPlaylistId):
        getMyplaylist = self.getMyPlaylist(myPlaylistId)
        dv = getMyplaylist["detail_ver"]
        payload = dict(items=[dict(gt=myPlaylistId, gl="", mv=dv, dv=dv, fn="")])
        r = self._post(self.JOOX_API_DOMAIN + self.JOOX_DEL_DIR_PATH, payload=payload)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def addTracksToMyPlaylist(self, songId, myPlaylistId):
        getMyplaylist = self.getMyPlaylist(myPlaylistId)
        dv = getMyplaylist["detail_ver"]
        payload = dict(items=[dict(gt=myPlaylistId, gl=songId, mv=dv, dv=dv, fn="")])
        r = self._post(self.JOOX_API_DOMAIN + self.JOOX_ADD_SONG_PATH, payload=payload)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def removeTracksFromMyPlaylist(self, songId, myPlaylistId):
        getMyplaylist = self.getMyPlaylist(myPlaylistId)
        dv = getMyplaylist["detail_ver"]
        payload = dict(items=[dict(gt=myPlaylistId, gl=songId, mv=dv, dv=dv, fn="")])

        r = self._post(self.JOOX_API_DOMAIN + self.JOOX_DEL_SONG_PATH, payload=payload)
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)

    @loggedIn
    def getTrackInfo(self, songId):
        params = dict(songid=songId)
        r = self._get(
            self.JOOX_API_DOMAIN + self.JOOX_GET_SONGINFO_PATH,
            params=params,
            withAuth=True,
        )
        if r.status_code != 200:
            raise ("Failed to fetch data.")
        return json.loads(r.text)


from datetime import datetime

import hashlib
import json


def encryptPassword(password):
    md5 = hashlib.md5()
    old_password = password.encode("utf-8")
    md5.update(old_password)
    new_password = md5.hexdigest()
    return new_password


def parseJSONP(data_jsonp):
    data_json = data_jsonp.split("(")[1].strip(")")
    return data_json


def log(text):
    print("[%s] %s" % (str(datetime.now()), text))